package com.ruoyi.project.reliability.mapper;

import java.util.List;
import java.util.Map;

import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.common.domain.DataEntity;
import com.ruoyi.project.reliability.domain.TAbnormalreport;

/**
 * 异常报告管理Mapper接口
 *
 * @author ruoyi
 * @date 2020-11-27
 */
public interface TAbnormalreportMapper
{
    /**
     * 查询异常报告管理
     *
     * @param id 异常报告管理ID
     * @return 异常报告管理
     */
    public TAbnormalreport selectTAbnormalreportById(Long id);

    /**
     * 查询异常报告管理列表
     *
     * @param tAbnormalreport 异常报告管理
     * @return 异常报告管理集合
     */
    @DataScope(deptAlias = "d")
    public List<TAbnormalreport> selectTAbnormalreportList(TAbnormalreport tAbnormalreport);

    /**
     * 新增异常报告管理
     *
     * @param tAbnormalreport 异常报告管理
     * @return 结果
     */
    public int insertTAbnormalreport(TAbnormalreport tAbnormalreport);

    /**
     * 修改异常报告管理
     *
     * @param tAbnormalreport 异常报告管理
     * @return 结果
     */
    public int updateTAbnormalreport(TAbnormalreport tAbnormalreport);

    /**
     * 删除异常报告管理
     *
     * @param id 异常报告管理ID
     * @return 结果
     */
    public int deleteTAbnormalreportById(Long id);

    /**
     * 批量删除异常报告管理
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTAbnormalreportByIds(Long[] ids);

    List<DataEntity> selectYearData(Map param);

    List<DataEntity> teamData(Map param);
}
